/*******************************************************************************

    ESRMETER_P872 (VERSO 7 SEGUIMENTOS CATODO COMUM)

    - SALVA AJUSTES DE ZERO NA EEPROM

    - FAIXA DE TRABALHO: 0 A 99 OHMS

    - EM 3 ESCALAS OPERANDO DE FORMA AUTOMTICA:

      A)0.00 A 0.99 OHMS
      B) 0.1 A 9.9  OHMS
      C) 10  A 99   OHMS

    - COM PIC 16F872

    AUTOR: CLAUDIO LRIOS

    DATA: 28/06/2022

    IMPORTANTE:
    DESCARREGUE OS CAPACITORES ANTES DE MEDIR SOB RISCO
    DE DANIFICAR O EQUIPAMENTO.
    USE SOMENTE PONTAS DE PROVAS (CABOS) DE BAIXA RESISTNCIA
    ELTRICA (ABAIXO DE 0,5 OHMS). O USO DE PONTAS DE PROVAS
    COM RESISTNCIA ACIMA DE 0,5 OMHS INVIABILIZA A ZERAGEM DO
    APARELHO E IMPEDE MEDIES PRECISAS DA ESCALA DE DEZENAS
    DE MILIOHMS.


    OBS. MONTAGEM  DE NATUREZA DIDTICA

    ESTE ARQUIVO  PARTE INTEGRANTE DO BLOG PICSOURCE


*******************************************************************************/

//=============================================================================/


#include <16F872.h>
#use delay(clock=4000000)
#fuses NOWDT,HS, PUT, NOPROTECT, BROWNOUT, NOCPD, NOLVP, NODEBUG

#ROM  0X2100 = {0,0,0,0,0,0}


//BYTE
#byte portc      = 0x07
#byte portb      = 0x06
#byte porta      = 0x05
#byte trisa      = 0x85
#byte trisb      = 0x86
#byte trisc      = 0x87
//#byte cmcon      = 0x19
#byte timer0     = 0x01
#byte option_reg = 0x81
#byte t1con      = 0x10
#byte adcon0     = 0x1f
#byte adcon1     = 0x9f
//#byte ansel      = 0x91
#byte adresh     = 0x1e
#byte adresl     = 0x9e

//BIT



#bit  escl    = 0x5.4  //escala baixa pino 6 - 100 ohms
#bit  escm    = 0x7.3  //escala mdia pino 14 - 1k
#bit  esch    = 0x5.2  //escala alta  pino 4 - 10k
#bit  desc    = 0x5.5  //descarga do capacitor cx - pino 7


#bit  kat     = 0x7.2  //rc2 pino 13 - multiplexao an/cat
#bit  ligar   = 0x7.0  //rc0 pino 11 - sada para ligar circuito
#bit  sw      = 0x7.1  //rc1 pino 12 - boto on/off/zerar



#bit  t0if    = 0x0b.2 //flag de interrupo do timer 0
#bit  t1if    = 0x0C.0 //flag do timer1
#bit go_done  = 0x1f.2 //flag de nicio de converso adc



#define VALOR_TIMEOUT 572 //tempo para 5 minutos em idle mode sendo: t= 5 * 60 * 1000000 /524288
#define VALOR_MIN_BAT 512 //equivalente a tenso de comparao 2,5 volts no pot1 de VBAT
//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTB 76543210
int8 const NUM[] = {     0B00111111,//0
                         0B00000110,//1
                         0B01011011,//2
                         0B01001111,//3
                         0B01100110,//4
                         0B01101101,//5
                         0B01111101,//6
                         0B00000111,//7
                         0B01111111,//8
                         0B01101111,//9
                                       };


 //variveis globais

 int16 piscab=0;
 int16 mtime=0;
 int16 res_esr=0;
 int16 zesrl=0;
 int16 zesrm=0;
 int16 zesrh=0;
 int8 deza;
 int8 una;
 int8 dez;
 int8 un;
 int8 old_esr=0;
 int1 pt1=0;
 int1 pt2=0;
 int1 pt1a=0;
 int1 pt2a=0;
 int1 f_lowb=0;//brilho reduzido
 int1 f_pisca=0;//pisca letra b da bateria descarregada
 int1 f_est=0;//flag de estouro de escalas
 int1 f_ok=0; //flag de leitura ok
 int1 f_zerar;//flag de zero


/******************************************************************************
/******************************************************************************
          ROTINA DE MULTIPLEXAO DOS DISPLAYS DE 7 SEGUIMENTOS

  Objetivo:
  Enviar dados para os displays.
  Regular brilho dos displays em modo 50 ou 100% (modo bateria com baixa carga).
  controlar transistor drives dos ctodos dos displays

******************************************************************************/

 void multiplexa(){
    static unsigned int8 vez;

    //define valores para intervalos da apresentao do 'b'
    #define VAL_PISCA1 1000 //0 a 999 = letra b acesa
    #define VAL_PISCA2 6000 //1000 a 5999 = leitura esr


    if(f_ok) {//temos nova leitura de ESR?
      f_ok=0;//sim
      pt1a=pt1;
      pt2a=pt2;
      deza=dez;//atualiza registrador de apresentao
      una=un;
    }
   // flag f_lowb indica quando bateria est descarregada
   if(f_lowb){
     piscab++;
     if(piscab>VAL_PISCA2)piscab=0;

     if(piscab<VAL_PISCA1){
       f_pisca=1;//intervalo letra B acesa
     }
     else{
       f_pisca=0;//intervalo letra B apagada

     }
   }
   else{
     f_pisca=0;
     piscab=0;
   }


   if(++vez>3) vez=0;//controle de multiplexao



    portb=0; //apaga seguimentos catodo
   // portb=0xff; //apaga todos seguimentos anodo


    switch (vez){

     case 0: {

       // CATODO
       if(pt2a)deza=deza | 0x80; else deza=deza & 0x7f;//determina o ponto
       if(f_est)deza=0x40;//trao do estouro de escalas
       if(f_pisca)deza=0; //modo pisca letra b apagando display de dezenas
       portb=deza;//envia para o display de 7 seg
       kat=1;

       /*ANODO
       if(pt2a)deza=deza & 0x7f; else deza=deza | 0x80;//determina o ponto
       if(f_est)deza=0xbf;//trao do estouro de escalas
       if(f_pisca)deza=0xff; //modo pisca letra b apagando display de dezenas
       portb=deza;//envia dezenas para o display de 7 seg
       kat=0;
       */

       if(!f_lowb) vez= vez+1;// pula prxima vez se brilho 100%
       break;}

     case 1: { //somente se flag de reduo de brilho estiver setado
       portb=0;//envia dado para apagar seguimentos (brilho = 50%) CATODO
       //portb=0XFF;//envia dado para apagar seguimentos (brilho = 50%) ANODO
       break;
     }

     case 2: {
       //catodo
       //pega casa da unidade
       if(pt1a)una=una | 0x80; else una=una & 0x7f;//determina o ponto
       if(f_est)una=0;//apaga display de unidades : trao do estouro de escalas
       if(f_pisca)una=0x7c; //modo pisca letra b enviando 'b' para unidades
       portb=una;//envia unidade para os seguimentos
       kat=0;

       /* anodo
       if(pt1a) una=una & 0x7f; else una=una | 0x80; //determina o ponto
       if(f_est)una=0xff;//apaga display de unidades : trao do estouro de escalas
       if(f_pisca)una=0x83; //modo pisca letra b enviando 'b' para unidades
       portb=una;//envia unidade para os seguimentos
       kat=1;
       */
       if(!f_lowb) vez= vez+1;// pula prxima vez se brilho 100%
       break;}

     case 3: { //somente se flag de reduo de brilho estiver setado
       portb=0;//envia dado para apagar seguimentos (brilho = 50%) CATODO
      // portb=0XFF;//envia dado para apagar seguimentos (brilho = 50%) ANODO
       break;
     }

    }


 }



/*****************************************************************************/
/******************************************************************************
   DELAY DE 500US COM MULTIPLEXAO DE DISPLAYS
******************************************************************************/
void delay_pep(){

  timer0=16;
  t0if=0;
  multiplexa();
  while(!t0if);//aguarda fim do tempo
}
/******************************************************************************
   ROTINA DELAY AJUSTAVEL COM MULTIPLEXAO DE DISPLAYS
******************************************************************************/
void delay_500us_adj(int8 v){

  while(v){
   v--;
   delay_pep();
  }
}

/******************************************************************************
           ROTINA DE LEITURA DE ESR: ESCALA L
******************************************************************************/

void leitura_esrl(){ //leitura ESR escala de 0.00 a 0.99 ohms
int8 a;
int16 soma;

   pt2=1;//liga ponto das dezenas
   pt1=0;//desliga ponto das unidades
   f_est=0;//zera estouro de escalas
   escl=1;//desliga escalas
   escm=1;
   esch=1;


//mdias de leituras
     soma=0;
     for(a=0;a<8;a++){
       desc=1; //liga descarga do capacitor sob teste
       res_esr=0;//zera
       delay_500us_adj(64);//tempo de descarga
       desc=0;//desliga descarga do cap sob teste
       escl=0;//liga escala baixa de esr
       delay_us(10);
       escl=1;//desliga escala baixa de esr
       go_done=1;//liga converso no pino 2 (an0)
       while(go_done);
       res_esr= (int16) adresh<<8;
       res_esr= res_esr  + adresl;
       soma= soma + res_esr;
       
       if(t1if){//verifica estouro do timer 1
       mtime++;//incrementa contador de tempo
       t1if=0;//reseta flag do timer 1
       }

     }
      res_esr= soma/8;//media

     desc=1;//liga descarga do capacitor sob teste
   //----------------------

   if(f_zerar){
   if(res_esr>=zesrl) res_esr= res_esr - zesrl; else res_esr=zesrl - res_esr ;
   }


}

/******************************************************************************
           ROTINA DE LEITURA DE ESR: ESCALA M
******************************************************************************/

void leitura_esrm(){ //leitura ESR escala de 1.0 a 9.9 ohms
int8 a;
int16 soma;

   pt2=0;//desliga ponto das dezenas
   pt1=1;//liga ponto das unidades
   escl=1;//desliga escalas
   esch=1;
   escm=1;

   //mdia de leituras
     soma=0;
     for(a=0;a<8;a++){

       desc=1; //liga descarga do capacitor sob teste
       res_esr=0;//zera
       delay_500us_adj(64);//tempo de descarga
       desc=0;//desliga descarga do cap sob teste
       escm=0;//liga escala mdia de esr
       delay_us(10);
       escm=1;//desliga escala mdia de esr
       go_done=1;//liga converso no pino 2 (an0)
       while(go_done);
       res_esr= (int16) adresh<<8;
       res_esr= res_esr  + adresl;
       soma= soma + res_esr;
       
       if(t1if){//verifica estouro do timer 1
       mtime++;//incrementa contador de tempo
       t1if=0;//reseta flag do timer 1
       }

     }
      res_esr= soma/8;//media

     desc=1;//liga descarga do capacitor sob teste
   //----------------------

   if(f_zerar){
     if(res_esr>=zesrm) res_esr= res_esr- zesrm; else res_esr=zesrm - res_esr;
   }

}

/******************************************************************************
           ROTINA DE LEITURA DE ESR: ESCALA H
******************************************************************************/

void leitura_esrh(){ //leitura ESR escala de 10 a 99 ohms
int8 a;
int16 soma;

   pt2=0;//desliga ponto das dezenas
   pt1=0;//desliga ponto das unidades
   escl=1;//desliga escalas
   escm=1;
   esch=1;


   //mdias de leituras
     soma=0;
     for(a=0;a<8;a++){
       desc=1; //liga descarga do capacitor sob teste
       res_esr=0;//zera
       delay_500us_adj(64);//tempo de descarga
       desc=0;//desliga descarga do cap sob teste
       esch=0;//liga escala alta de esr
       delay_us(10);
       esch=1;//desliga escala alta de esr
       go_done=1;//liga converso no pino 2 (an0)
       while(go_done);
       res_esr= (int16) adresh<<8;
       res_esr= res_esr  + adresl;
       soma= soma + res_esr;       

       if(t1if){//verifica estouro do timer 1
       mtime++;//incrementa contador de tempo
       t1if=0;//reseta flag do timer 1
       }

     }
      res_esr= soma/8;//media

     desc=1;//liga descarga do capacitor sob teste
   //----------------------

   if(f_zerar){
     if(res_esr>=zesrh) res_esr= res_esr- zesrh; else res_esr=zesrh - res_esr;
   }

}

//==============================================================================
//         ROTINA MAIN
//==============================================================================

void main() {
   option_reg=0x80; //timer0 /2 no pullup
   trisa=0b001011;//entradas (1) e saidas (0)
   trisb=0; //tudo sada para 7 seguimentos
   trisc=0b11110010;//sadas = rc0, rc2 , rc3

  // portb=0b10111111;//apaga seguimentos excet o 'g' anodo
   portb=0b01000000;//apaga seguimentos excet o 'g' catodo

   porta=0B111100;// descarga ligada e seletores de escala desligados
   portc=0B00001101;//travar ligado e escm=1 (desliga
   adcon0= 0b10000001;//ajusta para canal 0 adc e liga conversor adc
   adcon1= 0b10000100; //justificado a direita somente canais 0,1,3 vref+=vdd e vref-=gnd
   t1con=0b00110001;//liga timer1 internal prescaller/8

   while(!sw) delay_ms(50);
   //recupera ajuste de zero
   zesrl= (int16)read_eeprom(1)<<8;
   zesrl= zesrl +  read_eeprom(0);
   zesrm= (int16)read_eeprom(3)<<8;
   zesrm= zesrm +  read_eeprom(2);
   zesrh= (int16)read_eeprom(5)<<8;
   zesrh= zesrh +  read_eeprom(4);
   f_zerar=1;
//==============================================================================
//         LAO DE REPETIO PRINCIPAL
//==============================================================================

 for(;;){

       leitura_esrl();//leitura na escala de 0,00 a 0,99 ohms
       if(res_esr>99){//se houve estouro nesta escala faz
         leitura_esrm();//leitura na escala de 0,1 a 9.9 ohms
         if(res_esr>99){//se houve estouro nesta escala faz
          leitura_esrh();//leitura na escala de 10 a 99 ohms
          if(res_esr>99){//testa estouro da escala 10 a 99 ohms
           f_est=1;//sinaliza estouro de escalas
           res_esr=99;//evita erros na transformao para 7 seguimentos

         }
       }
     }

     //Zerar se pontas de provas em curto/ boto apertado
     if((pt2==1) && (sw==0)){//gerar valores para zerar
       zesrl=0;//condico inicial para nova zeragem
       zesrm=0;
       zesrh=0;
       f_zerar=0;
       leitura_esrl();//leitura na escala de 0,01 a 0,99 ohms
       zesrl=res_esr;//salva valor para zerar desta escala
       write_eeprom(0, zesrl);
       write_eeprom(1, zesrl>>8);
       leitura_esrm();//leitura na escala de 1 a 9,9 ohms
       zesrm=res_esr;//salva valor para zerar desta escala
       write_eeprom(2, zesrm);
       write_eeprom(3, zesrm>>8);
       leitura_esrh();//leitura na escala de 10 a 99 ohms
       zesrh=res_esr;//salva valor para zerar desta escala
       write_eeprom(4, zesrh);
       write_eeprom(5, zesrh>>8);
       f_zerar=1;//seta flag para indicar que foi feita operao de zerar
       res_esr=0;//zera para indicar mudana
     }


     dez=NUM[res_esr/10];//converte valor encontrado para decimal dezenas
    // dez=~dez ;//inverte se anodo
     un= NUM[res_esr%10];//converte valor encontrado para decimal unidades
    // un= ~un ; //inverte se anodo
     f_ok=1;//ativa flag de leitura ok


     //desliga aparelho apertando boto sw
     //precisa estar na escala 10-99 mostrando '-'
     if((sw==0) && (pt1==0) && (pt2==0) && (res_esr==99)){
       ligar=0;
       portb=0; //catodo
      // portb=0xff;//anodo
       for(;;);
     }


     //falta desligar por tempo 3 a 5 minutos sem mudana no valor
     if((res_esr> old_esr + 3) || (res_esr < old_esr - 3))mtime=0;
     old_esr= res_esr;

     if(t1if){//verifica estouro do timer 1
       mtime++;//incrementa contador de tempo
       t1if=0;//reseta flag do timer 1
     }

     if(mtime>VALOR_TIMEOUT){
       ligar=0;//desliga aparelho por tempo
       portb=0;//catodo
      // portb=0xff; //anodo
       for(;;);//loop eterno at acabar energia
     }

    //testa situao da carga da bateria
     adcon0=0; // desliga conversor adc
     adcon0= 0b10001001;//ajusta para canal 1 adc e liga conversor adc
     delay_us(50);
     go_done=1;//liga converso no pino 3 (an1) e testa vbat
     while(go_done);
     res_esr= (int16) adresh<<8;
     res_esr= res_esr  + adresl;
     if(res_esr< VALOR_MIN_BAT) f_lowb=1; else f_lowb=0; //se menor pisca 'b'
     desc=1;//liga descarga do capacitor sob teste
     adcon0=0; // desliga conversor adc
     adcon0= 0b10000001;//ajusta para canal 0 adc e liga conversor adc

     //delay para taxa de alterao dos displays = 3,3 x por segundo
   //  delay_500us_adj(250);//delay de aproximadamente 200 x 500us = 125 mseg
     delay_500us_adj(250);//delay de aproximadamente 200 x 500us = 125 mseg

 }//for(;;)

}//main()

